** This file is part of aITALC
** Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>
**-----------------------------------------------------------------------
** General aspect about numbering external particles by Diana
**  input particles get number -1,-2,-3,...,-m
** output particles get number -{m+1},...,-{n}


**                         / (-2)
** Typical then   (-1) ---X
**                         \ (-3)

** or

**   (-1) \ / (-3)
**         X
**   (-2) / \ (-4)

** We can understand the way of programming (negative signs are dropped
** out on comments)

***********************************************************************
\if "\import(maxexternalfermionindex)" eq "0" then
** Case of no fermions, so the definition of fermionchannel is empty ""
  \let(_fermionchannel,EMPTY)

***********************************************************************
\elif "\import(maxexternalfermionindex)" eq "1" then
** The complicated case:

  \if "\numberofexternallines()" eq "2" then
** Do nothing, is a self energy
    \let(_fermionchannel,UNKNOWN)

  \elif "\numberofexternallines()" eq "3" then
** We will consider the decay case 1->2 particles, so a fermion
** connecting lines 12 is T, 13 is U and 23 is S type of fermicurrent
    \if "\linefcount(-1)" eq "\linefcount(-2)" then
      \let(_fermionchannel,T)
    \elif "\linefcount(-1)" eq "\linefcount(-3)" then
      \let(_fermionchannel,U)
    \elif "\linefcount(-2)" eq "\linefcount(-3)" then
      \let(_fermionchannel,S)
    \else
      \let(_fermionchannel,WRONG)
    \endif

  \elif "\numberofexternallines()" eq "4" then
** We will consider the decay case 2->2 particles, so a fermion
** connecting lines 12 is S, 13 is T and 14 is U type of fermicurrent
** It also may happen than lines 12, are connected by a boson with 
** \linefcount = 0, but then, the fermion line connects 34 and we talk
** as well about a fermionchannel = S

** The definition of S,T and U doesn't change with respect to 2
** fermion currents on external legs

    \if "\linefcount(-1)" eq "\linefcount(-2)" then
      \let(_fermionchannel,S)
    \elif "\linefcount(-1)" eq "\linefcount(-3)" then
      \let(_fermionchannel,T)
    \elif "\linefcount(-1)" eq "\linefcount(-4)" then
      \let(_fermionchannel,U)
    \else
      \let(_fermionchannel,WRONG)
    \endif

  \elif "\numberofexternallines()" eq "5" then
** We will consider the case 2->3 particles, so a fermion
** connecting lines 12 is S, 13 is T and 15 is U type of fermicurrent
** The case of 14 should not be allowed

    \if "\linefcount(-1)" eq "\linefcount(-2)" then
      \let(_fermionchannel,S)
    \elif "\linefcount(-1)" eq "\linefcount(-3)" then
      \let(_fermionchannel,T)
    \elif "\linefcount(-1)" eq "\linefcount(-5)" then
      \let(_fermionchannel,U)
    \else
      \let(_fermionchannel,WRONG)
    \endif

  \else
      \let(_fermionchannel,UNKNOWN)

  \endif

************************************************************************
\elif "\import(maxexternalfermionindex)" eq "2" then
** Should be at least 4 ext particles with the same conf than for 1 FC

  \if "\numberofexternallines()" eq "4" then

    \if   "\linefcount(-1)" eq "\linefcount(-2)" then
      \let(_fermionchannel,S)
    \elif "\linefcount(-1)" eq "\linefcount(-3)" then
      \let(_fermionchannel,T)
    \elif "\linefcount(-1)" eq "\linefcount(-4)" then
      \let(_fermionchannel,U)
    \else
      \let(_fermionchannel,WRONG)
    \endif

  \elif "\numberofexternallines()" eq "5" then
** We will consider the case 2->3 particles, so a fermion
** connecting lines 12 is S, 13 is T and 15 is U type of fermicurrent
** The case of 14 should not be allowed

    \if   "\linefcount(-1)" eq "\linefcount(-2)" then
      \let(_fermionchannel,S)
    \elif "\linefcount(-1)" eq "\linefcount(-3)" then
      \let(_fermionchannel,T)
    \elif "\linefcount(-1)" eq "\linefcount(-5)" then
      \let(_fermionchannel,U)
    \else
      \let(_fermionchannel,WRONG)
    \endif

  \else
** We do not know what happen with 2 FermiCurrents and more than 5 ext legs.
    \let(_fermionchannel,UNKNOWN)

  \endif

\else
** We also do not imagine more than 3 FermiCurrents in ext legs
  \let(_fermionchannel,UNKNOWN)
\endif

** Before end, we create the first "memomenta{S,T or U}1" to be a signal in order
** to build the fermiontopnames.

\if not exist "memomenta\get(_fermionchannel)1" then
   \if exist "fermiontopnames" then
       \export(fermiontopnames,\import(fermiontopnames)\(,)\get(_fermionchannel))
   \else
       \export(fermiontopnames,\get(_fermionchannel))
   \endif
   \export(memomenta\get(_fermionchannel)1,\(CREATED))
\endif

\export(fermionchannel,\get(_fermionchannel))
\killvar(_fermionchannel)
** $Id: fermioncurrentanalysis.prg,v 1.3 2004/10/20 14:11:29 alorca Exp $
