C File <ics.f> for computing integrated cross sections following
C the Richardson extrapolation to Romberg integration with N=maxorder-1
C steps.
C This file is part of aITALC
C Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>

      subroutine ICS(
     &     setsqrtsman,setfracomega,setlimcost !input
     &     ,Ttot,Tfba,Etot,Efba)           !output

C Makes the integration of the DCS using the Romberg algorithm
C based on Richardson extrapolation

      implicit none
      integer maxorder
      parameter (maxorder=5)
      integer istep,iorder,ilevel,fullorder,sidepoints,j
      parameter (
     &     sidepoints=2**maxorder,
     &     fullorder=4*(maxorder+1)*maxorder
     &     )
      integer step(0:maxorder)
      double precision setsqrtsman,setcost,setfracomega,setlimcost
      double precision fun(4,-sidepoints:sidepoints),h(0:maxorder)
      double precision dcstree,dcsloop,dcsloop2
      double precision Tneg(4,0:maxorder-1,0:maxorder)
     &     ,Tpos(4,0:maxorder-1,0:maxorder)
     &     ,Tsum(4,0:maxorder-1,0:maxorder)
     &     ,Tdif(4,0:maxorder-1,0:maxorder)
     &     ,Esum(4,2:maxorder-1)
     &     ,Edif(4,2:maxorder-1)
     &     ,Ttot(4)
     &     ,Tfba(4)
     &     ,Etot(4)
     &     ,Efba(4)
      double precision limita,limitb,totalh
      double precision SOFTPHOTON
c      data limita,limitb /0d0,3.141592653589793d0/
      data Tneg,Tpos /fullorder*0d0,fullorder*0d0/
C-----------------------------------------------------------------------
C Making True zero our Tneg and Tpos, (no with data because of recalls)
      do istep=0,maxorder
        do iorder=0,maxorder-1
          do ilevel=1,4
            Tpos(ilevel,iorder,istep)=0d0
            Tneg(ilevel,iorder,istep)=0d0
            Tsum(ilevel,iorder,istep)=0d0
            Tdif(ilevel,iorder,istep)=0d0
          enddo
        enddo
      enddo
C-----------------------------------------------------------------------
C Computing the full step
      limita=-setlimcost
      limitb=+setlimcost
      totalh=(limitb-limita)
C-----------------------------------------------------------------------
C Doing the grid for selected points 2*sidepoints+1
      do j=-sidepoints,sidepoints
        setcost= j*totalh/(2*sidepoints)+(limita+limitb)/2
        call INIAITALC(setsqrtsman,setcost)
        call DCS(dcstree,dcsloop,dcsloop2)

        fun(1,j)=dcstree
        fun(2,j)=dcsloop
        fun(3,j)=SOFTPHOTON(setfracomega)*dcstree
        fun(4,j)=dcsloop2

C Dummy for testing functions
c        fun(1,j)=DSIN(setcost)
c        fun(2,j)=DEXP(setcost)
c        fun(3,j)=DEXP(-setcost)*DSIN(8*setcost**2)+1d0
c        fun(4,j)=3d0
c        print 11,fun(1,j),fun(2,j),fun(3,j),fun(4,j)
      enddo
C-----------------------------------------------------------------------
C Along next steps, we compute with the first call, the positive and
C negative range in cost, for the tree,loop,loop^2 and soft differential
C cross sections. The Romberg algorithm will converge fast enough if the
C functions are smooth. 

      h(0)=totalh/2
      step(0)=sidepoints

      do ilevel=1,4             !for tree,loop,loop2, and soft
        
        do istep=0,maxorder
C The negative and positive branches
          do j=-sidepoints,sidepoints,step(istep)
            if (j.EQ.-sidepoints) then
              Tneg(ilevel,0,istep)=Tneg(ilevel,0,istep)+fun(ilevel,j)
            elseif (j.EQ.sidepoints) then
              Tpos(ilevel,0,istep)=Tpos(ilevel,0,istep)+fun(ilevel,j)
            elseif (j.EQ.0) then
              Tneg(ilevel,0,istep)=Tneg(ilevel,0,istep)+fun(ilevel,j)
              Tpos(ilevel,0,istep)=Tpos(ilevel,0,istep)+fun(ilevel,j)
            elseif (j.LT.0) then
              Tneg(ilevel,0,istep)=Tneg(ilevel,0,istep)
     &             +2*fun(ilevel,j)
            elseif (j.GT.0) then
              Tpos(ilevel,0,istep)=Tpos(ilevel,0,istep)
     &             +2*fun(ilevel,j)
            endif
          enddo
          Tneg(ilevel,0,istep)=Tneg(ilevel,0,istep)*h(istep)/2
          Tpos(ilevel,0,istep)=Tpos(ilevel,0,istep)*h(istep)/2
          if (istep.LT.maxorder) then
            h(istep+1)=h(istep)/2
            step(istep+1)=step(istep)/2        
          endif
        enddo
C We have already computed the first bunch of sample data and done the
C zero order (T0), for the forward and backward directions.
C Now still are the extrapolations of higher order 

        do iorder=1,maxorder-1
          do istep=1,maxorder-iorder
            Tneg(ilevel,iorder,istep-1)= (
     &           4**iorder*Tneg(ilevel,iorder-1,istep)
     &                    -Tneg(ilevel,iorder-1,istep-1))
     &           /(4**iorder-1)
            Tpos(ilevel,iorder,istep-1)= (
     &           4**iorder*Tpos(ilevel,iorder-1,istep)
     &                    -Tpos(ilevel,iorder-1,istep-1))
     &           /(4**iorder-1)

            Tsum(ilevel,iorder,istep-1)= 
     &           +Tpos(ilevel,iorder,istep-1)
     &           +Tneg(ilevel,iorder,istep-1)
            Tdif(ilevel,iorder,istep-1)= 
     &           +Tpos(ilevel,iorder,istep-1)
     &           -Tneg(ilevel,iorder,istep-1)

c            print 9,'Tneg(',ilevel,iorder,istep-1,')=',Tneg(ilevel
c     &           ,iorder,istep-1)
c            print 9,'Tpos(',ilevel,iorder,istep-1,')=',Tpos(ilevel
c     &           ,iorder,istep-1)

c            print 9,'ICS (',ilevel,iorder,istep-1,')=',Tsum(ilevel
c     &           ,iorder,istep-1)
c            print 9,'FBA (',ilevel,iorder,istep-1,')=',Tdif(ilevel
c     &           ,iorder,istep-1)
          enddo                 !end of istep loop

          if (iorder.GT.1) then
            Esum(ilevel,iorder)=DABS(
     &           +Tsum(ilevel,iorder-1,istep-1)
     &           -Tsum(ilevel,iorder,istep-2))
            Edif(ilevel,iorder)=DABS(
     &           +Tdif(ilevel,iorder-1,istep-1)
     &           -Tdif(ilevel,iorder,istep-2))
c            print *,'Err(',ilevel,iorder,')=',Esum(ilevel,iorder)
c     &           ,+Tsum(ilevel,iorder-1,istep-1)
c     &           ,-Tsum(ilevel,iorder,istep-2)
          endif
        enddo                   !end of iorder loop

        Ttot(ilevel)=
     &       +Tpos(ilevel,maxorder-1,0)
     &       +Tneg(ilevel,maxorder-1,0)
        Tfba(ilevel)=
     &       +Tpos(ilevel,maxorder-1,0)
     &       -Tneg(ilevel,maxorder-1,0)

C Warnings if error are not getting reduced
        do iorder=2,maxorder-1
          if (Esum(ilevel,iorder).LT.Esum(ilevel,iorder)) then
            print *,'ics: Warning, Integral for ilevel='
     &           ,ilevel
     &           ,'perhaps not convergent'
          endif
        enddo
        Etot(ilevel)=Esum(ilevel,maxorder-1)
        Efba(ilevel)=Edif(ilevel,maxorder-1)

c        print 10,'Ttot(',ilevel,')',Ttot(ilevel),'+-',Etot(ilevel)
c        print 10,'Tfb (',ilevel,')',Tfba(ilevel),'+-',Efba(ilevel)
      enddo                     !of ilevel = tree,loop,loop2,soft
C-----------------------------------------------------------------------
 9    format (A5,3(I2),A2,1(1X,E22.16))
 10   format (A5,I1,2(A2,1X,E22.16,1X))
 11   format (4(E22.16,1X))
C-----------------------------------------------------------------------
      return
      end


C***********************************************************************
C The ICS8 is just a test routine with much higher accuracy in the
C integration due to the maxorder=8 of the algorithm.
C Takes of course much more time.
      subroutine ICS8(
     &     setsqrtsman,setfracomega,setlimcost !input
     &     ,Ttot,Tfba,Etot,Efba)           !output

C Makes the integration of the DCS using the Romberg algorithm
C based on Richardson extrapolation

      implicit none
      integer maxorder
      parameter(maxorder=8)
      integer istep,iorder,ilevel,fullorder,sidepoints,j
      parameter(sidepoints=2**maxorder
     &     ,fullorder=4*(maxorder+1)*maxorder)
      integer step(0:maxorder)
      double precision setsqrtsman,setcost,setfracomega,setlimcost
      double precision fun(4,-sidepoints:sidepoints),h(0:maxorder)
      double precision dcstree,dcsloop,dcsloop2
      double precision Tneg(4,0:maxorder-1,0:maxorder)
     &     ,Tpos(4,0:maxorder-1,0:maxorder)
     &     ,Tsum(4,0:maxorder-1,0:maxorder)
     &     ,Tdif(4,0:maxorder-1,0:maxorder)
     &     ,Esum(4,2:maxorder-1)
     &     ,Edif(4,2:maxorder-1)
     &     ,Ttot(4)
     &     ,Tfba(4)
     &     ,Etot(4)
     &     ,Efba(4)
      double precision limita,limitb,totalh
      double precision SOFTPHOTON
c      data limita,limitb /0d0,3.141592653589793d0/
      data Tneg,Tpos /fullorder*0d0,fullorder*0d0/
C-----------------------------------------------------------------------
C Making True zero our Tneg and Tpos, (no with data because of recalls)
      do istep=0,maxorder
        do iorder=0,maxorder-1
          do ilevel=1,4
            Tpos(ilevel,iorder,istep)=0d0
            Tneg(ilevel,iorder,istep)=0d0
            Tsum(ilevel,iorder,istep)=0d0
            Tdif(ilevel,iorder,istep)=0d0
          enddo
        enddo
      enddo
C-----------------------------------------------------------------------
C Computing the full step
      limita=-setlimcost
      limitb=+setlimcost
      totalh=(limitb-limita)
C-----------------------------------------------------------------------
C Doing the grid for selected points 2*sidepoints+1
      do j=-sidepoints,sidepoints
        setcost= j*totalh/(2*sidepoints)+(limita+limitb)/2
        call INIAITALC(setsqrtsman,setcost)
        call DCS(dcstree,dcsloop,dcsloop2)

        fun(1,j)=dcstree
        fun(2,j)=dcsloop
        fun(3,j)=SOFTPHOTON(setfracomega)*dcstree
        fun(4,j)=dcsloop2

C Dummy for testing functions
c        fun(1,j)=DSIN(setcost)
c        fun(2,j)=DEXP(setcost)
c        fun(3,j)=DEXP(-setcost)*DSIN(8*setcost**2)+1d0
c        fun(4,j)=3d0
c        print *,fun(1,j),fun(2,j),fun(3,j),fun(4,j)
      enddo
C-----------------------------------------------------------------------
C Along next steps, we compute with the first call, the positive and
C negative range in cost, for the tree,loop,loop^2 and soft differential
C cross sections. The Romberg algorithm will converge fast enough if the
C functions are smooth. 

      h(0)=totalh/2
      step(0)=sidepoints

      do ilevel=1,4             !for tree,loop,loop2, and soft
        
        do istep=0,maxorder
C The negative and positive branches
          do j=-sidepoints,sidepoints,step(istep)
            if (j.EQ.-sidepoints) then
              Tneg(ilevel,0,istep)=Tneg(ilevel,0,istep)+fun(ilevel,j)
            elseif (j.EQ.sidepoints) then
              Tpos(ilevel,0,istep)=Tpos(ilevel,0,istep)+fun(ilevel,j)
            elseif (j.EQ.0) then
              Tneg(ilevel,0,istep)=Tneg(ilevel,0,istep)+fun(ilevel,j)
              Tpos(ilevel,0,istep)=Tpos(ilevel,0,istep)+fun(ilevel,j)
            elseif (j.LT.0) then
              Tneg(ilevel,0,istep)=Tneg(ilevel,0,istep)
     &             +2*fun(ilevel,j)
            elseif (j.GT.0) then
              Tpos(ilevel,0,istep)=Tpos(ilevel,0,istep)
     &             +2*fun(ilevel,j)
            endif
          enddo
          Tneg(ilevel,0,istep)=Tneg(ilevel,0,istep)*h(istep)/2
          Tpos(ilevel,0,istep)=Tpos(ilevel,0,istep)*h(istep)/2
          if (istep.LT.maxorder) then
            h(istep+1)=h(istep)/2
            step(istep+1)=step(istep)/2        
          endif
        enddo
C We have already computed the first bunch of sample data and done the
C zero order (T0), for the forward and backward directions.
C Now still are the extrapolations of higher order 

        do iorder=1,maxorder-1
          do istep=1,maxorder-iorder
            Tneg(ilevel,iorder,istep-1)= (
     &           4**iorder*Tneg(ilevel,iorder-1,istep)
     &                    -Tneg(ilevel,iorder-1,istep-1))
     &           /(4**iorder-1)
            Tpos(ilevel,iorder,istep-1)= (
     &           4**iorder*Tpos(ilevel,iorder-1,istep)
     &                    -Tpos(ilevel,iorder-1,istep-1))
     &           /(4**iorder-1)

            Tsum(ilevel,iorder,istep-1)= 
     &           +Tpos(ilevel,iorder,istep-1)
     &           +Tneg(ilevel,iorder,istep-1)
            Tdif(ilevel,iorder,istep-1)= 
     &           +Tpos(ilevel,iorder,istep-1)
     &           -Tneg(ilevel,iorder,istep-1)

c            print 9,'Tneg(',ilevel,iorder,istep-1,')=',Tneg(ilevel
c     &           ,iorder,istep-1)
c            print 9,'Tpos(',ilevel,iorder,istep-1,')=',Tpos(ilevel
c     &           ,iorder,istep-1)

c            print 9,'ICS (',ilevel,iorder,istep-1,')=',Tsum(ilevel
c     &           ,iorder,istep-1)
c            print 9,'FBA (',ilevel,iorder,istep-1,')=',Tdif(ilevel
c     &           ,iorder,istep-1)
          enddo                 !end of istep loop

          if (iorder.GT.1) then
            Esum(ilevel,iorder)=DABS(
     &           +Tsum(ilevel,iorder-1,istep-1)
     &           -Tsum(ilevel,iorder,istep-2))
            Edif(ilevel,iorder)=DABS(
     &           +Tdif(ilevel,iorder-1,istep-1)
     &           -Tdif(ilevel,iorder,istep-2))
c            print *,'Err(',ilevel,iorder,')=',Esum(ilevel,iorder)
c     &           ,+Tsum(ilevel,iorder-1,istep-1)
c     &           ,-Tsum(ilevel,iorder,istep-2)
          endif
        enddo                   !end of iorder loop

        Ttot(ilevel)=
     &       +Tpos(ilevel,maxorder-1,0)
     &       +Tneg(ilevel,maxorder-1,0)
        Tfba(ilevel)=
     &       +Tpos(ilevel,maxorder-1,0)
     &       -Tneg(ilevel,maxorder-1,0)

C Warnings if error are not getting reduced
        do iorder=2,maxorder-1
          if (Esum(ilevel,iorder).LT.Esum(ilevel,iorder)) then
            print *,'ics: Warning, Integral for ilevel='
     &           ,ilevel
     &           ,'perhaps not convergent'
          endif
        enddo
        Etot(ilevel)=Esum(ilevel,maxorder-1)
        Efba(ilevel)=Edif(ilevel,maxorder-1)

c        print 10,'Ttot(',ilevel,')',Ttot(ilevel),'+-',Etot(ilevel)
c        print 10,'Tfb (',ilevel,')',Tfba(ilevel),'+-',Efba(ilevel)
      enddo                     !of ilevel = tree,loop,loop2,soft
C-----------------------------------------------------------------------
 9    format (A5,3(I2),A2,1(1X,F17.15))
 10   format (A5,I1,2(A2,1X,E22.16,1X))

      return
      end

CC $Id: ics.f,v 1.8 2004/10/07 15:05:11 alorca Exp $
